import logging

from sqlalchemy import func
from ec.model.outputindicator import ECOutputindicator
from sqlalchemy.exc import IntegrityError
from ecimporter.lib.ecimport import ecimport
from ecimporter.lib.transfer_bewec_ec import WrongStateError
from ecimporter.lib.transfer_bewec_ec import get_editor_mapping
from ecimporter.lib.transfer_bewec_ec import transfer_model, ConfigValueError
from ecimporter.lib.data_validation import report_differences
from ecimporter.lib.data_validation import check_input_differences
from pyramid.view import view_config

log = logging.getLogger(__name__)


@view_config(route_name='import', renderer="json")
def my_view(request):
    """
    checks incoming fields for completeness
    retrieves additional data of editors
    does the actual import to ec

    :param request:
    :return: JSON - Response containing status, payload etc.
    """

    # Add a dummy translate attribute. This is later needed if there are any
    # validation errors on the form that are translated.
    request.translate = lambda x: x

    # validation is done here
    status_fail, msg_data = check_input_differences(request.body)
    if status_fail:
        return response(**msg_data)

    # the real import is done here
    try:
        bearbeiter_uid, bearbeiter_gid = get_editor_mapping(request)
        model = transfer_model(request.body, bearbeiter_uid,
                               bearbeiter_gid)
        ecimport(model, request)
    except (ConfigValueError, WrongStateError, AttributeError) as e:
        log.error(e.message)
        return response('fail', title=e.message)
    except IntegrityError as e:
        log.error(e.message)
        return response('fail', title=e.message)
    except Exception as e:
        log.exception("")
        return response('error', message=e.message)
    else:
        return response('success')


@view_config(route_name='course', renderer="json")
def get_course(request):
    """
    returns a course and the number of participations

    :param request:
    :return: JSON - Response containing status, payload etc.
    """
    if not request.GET.get('id'):
        return response('failure', message="ID parameter is required")

    out = {}
    course_pk = int(request.GET.get('id'))
    result = request.db.query(ECOutputindicator.course_id, func.count(ECOutputindicator.id).label('course'))\
        .filter(ECOutputindicator.course_id == course_pk )\
        .group_by(ECOutputindicator.course_id)
    if result.first():
        item = result.one()
        out[course_pk] = {'count': item[1]}
        return response('success', payload=out)
    else:
        out[course_pk] = {'count': 0}
        return response('failure', message="No course with this ID found", payload=out)


def response(status, title=None, message=None, payload=None):
    """
    Generates the response object which is serilized to JSON
    :param status:
    :param title:
    :param message:
    :param payload:
    :return: JSON
    """
    payload = payload or {}
    if status != 'success':
        payload['title'] = title
    return {
        'status': status,
        'message': message,
        'data': payload
    }
